program TestQuicksort;

{$APPTYPE CONSOLE}

{ Deplhi 5 }

uses 

   SysUtils;

const
   NMAX = 1024;
   NTEST = 1000;   
   RECLABEL = 'value ';

{ example of item }
type item = record
               Key: integer;
               Data: string;
            end;

var
   A: array [1..NMAX] of item;
   N: integer;
   I, J: integer;
   S: string;
   IsOK: boolean;

procedure ResetData();

   Var I: integer;
       S: string;

   begin
      for I := 1 to N do
      begin
         S := IntToStr(I);
         A[I].Data := RECLABEL + S;
      end;
   end;

procedure ResetKey();

   Var I, R: integer;

   begin
      for I := 1 to N do
      begin
         R := Random(N);
         if R = 0 then R := 1;
         A[I].Key := R
      end;
   end;

function GetValue(V: string): integer;

var
   L: integer;
   K: integer;
   S: string;

   begin
      K := Length(RECLABEL);
      L := Length(V);
      if L >= (K + 1) then
      begin
         S := Copy(V, (K + 1) , L - K);
         GetValue := StrToInt(S);
      end
      else
      begin
         GetValue := 0;
      end;

      exit;
   end;

{ comparison of A[I] and A[J] with respect of A[I].Key = A[J].Key }
{ LT = 'LowerThan' }
function LT(I, J: integer): boolean;

var
   X, Y: integer;

   begin
      { fast exit with true for A[I].Key < A[J].Key }
      if A[I].Key < A[J].Key then
      begin
         LT := true;
         exit;
      end;

      { fast exit with false for A[I].Key > A[J].Key }
      if A[I].Key > A[J].Key then
      begin
         LT := false;
         exit;
      end;

      { here is A[I].Key = A[J].Key }
      X := GetValue(A[I].Data);
      Y := GetValue(A[J].Data);

      if X < Y then
      begin
         LT := True;
         exit;
      end;

      LT := False;
      exit
   end;

procedure OutputArray(S: string);

   var I: integer;

   begin
      writeln(S);
      writeln('content of array:');
      for I:=1 to N do
      begin
         write('[');
         write(I);
         write(']');
         write(' Key = ');
         write(A[I].Key);
         write(', Data = ');
         write(A[I].Data);
         writeln;
      end;
      writeln;
   end;

{ see Niklaus Wirth, "Algorithmen und Datenstrukturen" }
procedure quicksort();

   procedure sort(L, R: integer);
   
      Var I, J: integer;
          X, W: item;

      begin
         I := L;
         J := R;
         X := A[(L + R) div 2];

         repeat
            while A[I].Key < X.Key do I := I + 1;
            while X.Key < A[J].Key do J := J - 1;

            if I <= J then
            begin
               W := A[I];
               A[I] := A[J];
               A[J] := W;

               I := I + 1;
               J := J - 1;
            end;

         until I > J;

         if L < J then sort(L, J);
         if I < R then sort(I, R);
      end;

   begin
      if N >= 1 then sort(1, N);
   end;

{ This is the first thing, that comes to mind. }
{ It did not make it :-( }
procedure stable_quicksort_NO();

   { comparison of A[I] and A[J] with respect of A[I].Key = A[J].Key }
   { LT = 'LowerThan' }
   function LT_loc(I, J: integer): boolean;

      begin
         { fast exit with true for A[I].Key < A[J].Key }
         if A[I].Key < A[J].Key then
         begin
            LT_loc := true;
            exit;
         end;

         { fast exit with false for A[I].Key > A[J].Key }
         if A[I].Key > A[J].Key then
         begin
            LT_loc := false;
            exit;
         end;

         { here is A[I].Key = A[J].Key }
         if I < J then
         begin
            LT_loc := true;
            exit;
         end;

         { here is I >= J and A[I].Key = A[J].Key }
         LT_loc := false;
         exit;
      end;

   { trace the middle }
   { Cave! The middle M moves only, when ((I <> J) And ((M = I) Or (M = J))) }
   procedure trace(var M: integer; I, J: integer);

      begin
         { middle is to be swapped? }
         if I <> J then
         begin
            if M = I then
               M := J
            else
               if M = J then M := I;
         end;
      end;

   { patched }
   procedure sort(L, R: integer);
   
      Var I, J, M: integer;
          W: item;

      begin
         I := L;
         J := R;

         { middle }
         M := (L + R) div 2;
 
         repeat
            { comparisons for A[] }
            while LT_loc(I, M) do I := I + 1;
            while LT_loc(M, J) do J := J - 1;

            { here is A[I] >= A[M] >= A[J] }

            if I <= J then
            begin
               { trace the middle }
               trace(M, I, J);

               W := A[I];
               A[I] := A[J];
               A[J] := W;

               I := I + 1;
               J := J - 1;
            end;

         until I > J;

         if L < J then sort(L, J);
         if I < R then sort(I, R);
      end;

   begin
      if N >= 1 then sort(1, N);
   end;

{ This one should do it. }
procedure stable_quicksort_that_needs_time();

   procedure sort(L, R: integer);
   
      Var I, J, K: integer;
          X, V, W: item;

      begin
         I := L;
         J := R;
         X := A[(L + R) div 2];

         repeat
            while A[I].Key < X.Key do I := I + 1;
            while X.Key < A[J].Key do J := J - 1;

            { here is A[I].Key >= X.Key >= A[J].Key }

            if I <= J then
            begin
               if A[I].Key <> A[J].Key then
               begin
                  { here is (I + 1) <= J }
                  W := A[I];
                  A[I] := A[J];  

                  { go forward through A[K] from (I + 1) to J }
                  { and swap A[K] and W wherever W.Key = A[K].KEy }
                  for K := (I + 1) to J do
                  begin
                     if W.Key = A[K].Key then
                     begin
                        { swap A[K] and W }
                        V := A[K];
                        A[K] := W;
                        W := V;
                     end;
                  end;

                  { swap A[J] and W }
                  V := A[J];
                  A[J] := W;
                  W := V;

                  { go backward through A[K] from (J - 1) to I }
                  { and swap A[K] and W wherever W.Key = A[K].KEy }
                  for K := (J - 1) downto I do
                  begin
                     if W.Key = A[K].Key then
                     begin
                        { swap A[K] and W }
                        V := A[K];
                        A[K] := W;
                        W := V;
                     end;
                  end;
               end;

               I := I + 1;
               J := J - 1;
            end;

         until I > J;

         if L < J then sort(L, J);
         if I < R then sort(I, R);
      end;

   begin
      if N >= 1 then sort(1, N);
   end;

procedure stable_quicksort_that_needs_memory();

var
   Index: array of integer;
   H: integer;

   { trace the middle }
   { Cave! The middle M moves only, when ((I <> J) And ((M = I) Or (M = J))) }
   procedure trace(var M: integer; I, J: integer);

      begin
         { middle is to be swapped? }
         if I <> J then
         begin
            if M = I then
               M := J
            else
               if M = J then M := I;
         end;
      end;

   { comparison of A[I] and A[J] with respect of A[I].Key = A[J].Key }
   { LT = 'LowerThan' }
   function LT_loc(I, J: integer): boolean;

      begin
         { fast exit with true for A[I].Key < A[J].Key }
         if A[I].Key < A[J].Key then
         begin
            LT_loc := true;
            exit;
         end;

         { fast exit with false for A[I].Key > A[J].Key }
         if A[I].Key > A[J].Key then
         begin
            LT_loc := false;
            exit;
         end;

         { here is A[I].Key = A[J].Key }
         if Index[I] < Index[J] then
         begin
            LT_loc := true;
            exit;
         end;

         { here is Index[I] >= Index[J] and A[I].Key = A[J].Key }
         LT_loc := false;
         exit;
      end;

   procedure sort(L, R: integer);
   
      Var I, J, K, M: integer;
          W: item;

      begin
         I := L;
         J := R;
         M := (L + R) div 2;

         repeat
            while LT_loc(I, M) do I := I + 1;
            while LT_loc(M, J) do J := J - 1;

            if I <= J then
            begin
               { trace the middle }
               trace(M, I, J);

               W := A[I];
               A[I] := A[J];
               A[J] := W;

               K := Index[I];
               Index[I] := Index[J];
               Index[J] := K;

               I := I + 1;
               J := J - 1;
            end;

         until I > J;

         if L < J then sort(L, J);
         if I < R then sort(I, R);
      end;

   begin
      if N >= 1 then
      begin
         { allocate Index for use in LT_loc }
         { Cave! Dynamic arrays begin with 0 and end with (dim - 1) }
         SetLength(Index, N + 1);

         { fill index }
         Index[0] := 0;
         for H := 1 to N do
         begin
            Index[H] := H;
         end;

         sort(1, N);

         { free index }
         SetLength(Index, 0);
      end;
   end;

begin
   IsOK := True;

   randomize;

   for I:= 1 to NTEST do
   begin
      S := IntToStr(I);
      write('Test No. ' + S);

      { dimension }
      N := random(NMAX);
      if N = 0 then N := 1;

      S := IntToStr(N);
      write(', Dimension ' + S);

      { test data in A[...] (random) }
      ResetKey;
      ResetData;

      if NTEST = 1 then
      begin
         writeln;
         OutputArray('(origin)');
      end;

{     Just try it yourself!
      quicksort;
      stable_quicksort_NO;
      stable_quicksort_that_needs_memory;
}

      stable_quicksort_that_needs_time;

      if NTEST = 1 then
      begin
         OutputArray('(result)');
      end;

      { check stable_quicksort }
      if N >= 2 then
      begin
         for J := 2 to N do
         begin
            if not LT(J - 1, J) then
            begin
               IsOK := False;
            end;
         end;
      end;

      if IsOK then
      begin
         writeln('...OK!');
      end
      else
      begin
         writeln('...Error!');
         OutputArray('(faulty)');
         exit;
      end;
   end;

   if not IsOK then
   begin
      writeln('There was an error!');
   end;
end.

